/* $Id: app.c,v 1.10 1997/09/15 14:52:30 wtcorrea Exp wtcorrea $ */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "app.h"
#include "graphics.h"

#define MAX_OBJS 1000
#define MAX_POINTS 50

typedef struct {
    int w, h;			/* width and height */
    int x1, y1;			/* location of last button1 click */
    int nclicks1;		/* number of button1 clicks */
    int tool;			/* tool currently in use */
} win_t;

typedef struct {
    obj_t objs[MAX_OBJS];
    int n;
} displist_t;

static win_t appwin;
static displist_t displist;

static attr_t curattr = {	/* current attributes */
    0.0, 0.0, 0.0, 1.0,		/* line color */
    1,				/* line width */
    0, 0, 0x0000,		/* line stipple flag, factor, and pattern */
    1,				/* boundary flag */
    1, 1.0, 0.0, 0.0, 1.0	/* fill flag and color */
};

static attr_t tmpattr = {	/* temporary attributes for rubberband */
    0.0, 0.0, 0.0, 1.0,		/* line color */
    1,				/* line width */
    1, 2, 0xAAAA,		/* line stipple flag, factor, and pattern */
    1,				/* boundary flag */
    0, 0.0, 0.0, 0.0, 1.0	/* fill flag and color */
};

int redraw;

void app_click1(int x, int y, int s)
{
/* test if shift was pressed */
#define shift(s) ((s & 1) == 1)

    y = appwin.h - y - 1;
    appwin.x1 = x, appwin.y1 = y;
    switch (appwin.tool) {
    case TOOL_CURSOR:
	{
	    int i, found;
	    obj_t *obj;

	    if (!shift(s)) {
		for (i = 0; i < displist.n; i++) {
		    displist.objs[i].selected = 0;
		}
	    }
	    for (i = displist.n - 1, found = 0; !found && i >= 0; i--) {
		obj = &displist.objs[i];
		if (obj_pick(obj, x, y)) {
		    found = 1;
		}
	    }
	    if (found) {
		if (shift(s)) {
		    obj->selected = !obj->selected;
		} else {
		    obj->selected = 1;
		}
	    }
	    redraw = 1;
	}
	break;
    case TOOL_LINE:
	{
	    obj_t *obj;

	    if (appwin.nclicks1 == 0) {
		if (displist.n == MAX_OBJS) {
		    fprintf(stderr, "Too many objects.\n");
		    return;
		}
		obj = &displist.objs[displist.n];
		obj->type = OBJ_LINE;
		obj->data = line_new(x, y, x, y);
		obj->attr = tmpattr;
		displist.n++;
		appwin.nclicks1++;
	    } else {
		obj = &displist.objs[displist.n - 1];
		line_lastpt(obj->data, x, y);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
	    }
	    redraw = 1;
	}
	break;
    case TOOL_POLYLINE:
	{
	    polyline_t *p;
	    obj_t *obj;

	    if (appwin.nclicks1 == 0) {
		if (displist.n == MAX_OBJS) {
		    fprintf(stderr, "Too many objects.\n");
		    return;
		}
		obj = &displist.objs[displist.n];
		p = polyline_new(MAX_POINTS);
		obj->type = OBJ_POLYLINE;
		obj->data = p;
		polyline_addpt(p, x, y);
		polyline_addpt(p, x, y);
		obj->attr = tmpattr;
		displist.n++;
		appwin.nclicks1++;
	    } else {
		obj = &displist.objs[displist.n - 1];
		p = obj->data;
		if (polyline_npts(p) < MAX_POINTS) {
		    polyline_addpt(p, x, y);
		    appwin.nclicks1++;
		} else {
		    polyline_lastpt(p, x, y);
		    obj->data = polyline_clone(p);
		    polyline_free(p);
		    obj->attr = curattr;
		    appwin.nclicks1 = 0;
		}
	    }
	    redraw = 1;
	}
	break;
    case TOOL_POLYGON:
	{
	    polygon_t *p;
	    obj_t *obj;

	    if (appwin.nclicks1 == 0) {
		if (displist.n == MAX_OBJS) {
		    fprintf(stderr, "Too many objects.\n");
		    return;
		}
		obj = &displist.objs[displist.n];
		p = polygon_new(MAX_POINTS);
		obj->type = OBJ_POLYGON;
		obj->data = p;
		polygon_addpt(p, x, y);
		polygon_addpt(p, x, y);
		obj->attr = tmpattr;
		displist.n++;
		appwin.nclicks1++;
	    } else {
		obj = &displist.objs[displist.n - 1];
		p = obj->data;
		if (polygon_npts(p) < MAX_POINTS) {
		    polygon_addpt(p, x, y);
		    appwin.nclicks1++;
		} else {
		    polygon_lastpt(p, x, y);
		    obj->data = polygon_clone(p);
		    polygon_free(p);
		    obj->attr = curattr;
		    appwin.nclicks1 = 0;
		}
	    }
	    redraw = 1;
	}
	break;
    case TOOL_TRIANGLE:
	{
	    triangle_t *t;
	    obj_t *obj;

	    switch (appwin.nclicks1) {
	    case 0:
		if (displist.n == MAX_OBJS) {
		    fprintf(stderr, "Too many objects.\n");
		    return;
		}
		obj = &displist.objs[displist.n];
		t = triangle_new();
		obj->type = OBJ_TRIANGLE;
		obj->data = t;
		triangle_setv(t, 0, x, y);
		triangle_setv(t, 1, x, y);
		triangle_setv(t, 2, x, y);
		obj->attr = tmpattr;
		displist.n++;
		appwin.nclicks1++;
		break;
	    case 1:
		obj = &displist.objs[displist.n - 1];
		triangle_setv(obj->data, 1, x, y);
		appwin.nclicks1++;
		break;
	    case 2:
		obj = &displist.objs[displist.n - 1];
		triangle_setv(obj->data, 2, x, y);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
		break;
	    default:
		fprintf(stderr, "Internal error.\n");
		assert(0);
		break;
	    }
	    redraw = 1;
	}
	break;
    case TOOL_RECT:
	{
	    obj_t *obj;

	    if (appwin.nclicks1 == 0) {
		if (displist.n == MAX_OBJS) {
		    fprintf(stderr, "Too many objects.\n");
		    return;
		}
		obj = &displist.objs[displist.n];
		obj->type = OBJ_RECT;
		obj->data = rect_new(x, y, x, y);
		obj->attr = tmpattr;
		displist.n++;
		appwin.nclicks1++;
	    } else {
		obj = &displist.objs[displist.n - 1];
		rect_lastpt(obj->data, x, y);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
	    }
	    redraw = 1;
	}
	break;
    default:
	fprintf(stderr, "app_click1: not implemented yet...\n");
	return;
    }
}

void app_dclick1(int x, int y, int s)
{
    if (appwin.tool != TOOL_CURSOR) {
	y = appwin.h - y - 1;
	appwin.x1 = x, appwin.y1 = y;
    }
    switch (appwin.tool) {
    case TOOL_CURSOR:
	app_click1(x, y, s);
	break;
    case TOOL_LINE:
	break;
    case TOOL_POLYLINE:
	{
	    obj_t *obj;
	    polyline_t *p;

	    if (appwin.nclicks1 > 0) {
		obj = &displist.objs[displist.n - 1];
		p = obj->data;
		polyline_delpt(p);
		polyline_lastpt(p, x, y);
		obj->data = polyline_clone(p);
		polyline_free(p);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
	    }
	    redraw = 1;
	}
	break;
    case TOOL_POLYGON:
	{
	    obj_t *obj;
	    polygon_t *p;

	    if (appwin.nclicks1 > 0) {
		obj = &displist.objs[displist.n - 1];
		p = obj->data;
		polygon_delpt(p);
		polygon_lastpt(p, x, y);
		obj->data = polygon_clone(p);
		polygon_free(p);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
	    }
	    redraw = 1;
	}
	break;
    case TOOL_TRIANGLE:
	break;
    case TOOL_RECT:
	break;
    default:
	fprintf(stderr, "app_dclick1: not implemented yet...\n");
	return;
    }
}

void app_delete(void)
{
    int i, j;
    obj_t tmp, *obj;

    i = 0;
    while (i < displist.n) {
	if (displist.objs[i].selected) {
	    tmp = displist.objs[i];
	    for (j = i; j < displist.n - 1; j++) {
		displist.objs[j] = displist.objs[j + 1];
	    }
	    obj = &displist.objs[displist.n - 1];
	    *obj = tmp;
	    obj_free_data(obj);
	    obj->type = OBJ_NONE;
	    obj->data = NULL;
	    obj->attr = curattr;
	    obj->selected = 0;
	    displist.n--;
	} else {
	    i++;
	}
    }
    redraw = 1;
}

void app_motion1(int x, int y)
{
    obj_t *obj;
    int i, dx, dy;

    if (appwin.nclicks1 > 0) {
	app_motion(x, y);
    } else {
	y = appwin.h - y - 1;
	dx = x - appwin.x1, dy = y - appwin.y1;
	for (i = 0; i < displist.n; i++) {
	    obj = &displist.objs[i];
	    if (obj->selected) {
		redraw = 1;
		obj_move(obj, dx, dy);
	    }
	}
	appwin.x1 = x, appwin.y1 = y;
    }
}

void app_motion(int x, int y)
{
    obj_t *obj;

    y = appwin.h - y - 1;
    if (appwin.nclicks1 > 0) {
	obj = &displist.objs[displist.n - 1];
	switch (appwin.tool) {
	case TOOL_CURSOR:
	    break;
	case TOOL_LINE:
	    line_lastpt(obj->data, x, y);
	    break;
	case TOOL_POLYLINE:
	    polyline_lastpt(obj->data, x, y);
	    break;
	case TOOL_POLYGON:
	    polygon_lastpt(obj->data, x, y);
	    break;
	case TOOL_TRIANGLE:
	    triangle_setv(obj->data, appwin.nclicks1, x, y);
	    break;
	case TOOL_RECT:
	    rect_lastpt(obj->data, x, y);
	    break;
	default:
	    fprintf(stderr, "app_motion: not implemented yet...\n");
	    return;
	}
	redraw = 1;
    }
}

void app_print(void)
{
    FILE *fp;

    fp = fopen("output.ps", "w");
    assert(fp);
    fprintf(fp, "%%!PS-Adobe-1.0\n");

    fprintf(fp, "0 0 moveto\n");
    fprintf(fp, "0 %d lineto\n", appwin.h - 1);
    fprintf(fp, "%d %d lineto\n", appwin.w - 1, appwin.h - 1);
    fprintf(fp, "%d 0 lineto\n", appwin.w - 1);
    fprintf(fp, "0 0 lineto\n");

    fprintf(fp, "stroke\n");
    fprintf(fp, "showpage\n");
    fclose(fp);
}

void app_new(int w, int h)
{
    int i;
    obj_t *obj;

    appwin.w = w, appwin.h = h;
    obj = &displist.objs[0];
    for (i = 0; i < displist.n; i++) {
	free(obj++);
    }
    obj = &displist.objs[0];
    for (i = 0; i < MAX_OBJS; i++) {
	obj->type = OBJ_NONE;
	obj->data = NULL;
	obj->attr = curattr;
	obj->selected = 0;
	obj++;
    }
    displist.n = 0;
    gr_set_window(-0.5, (double) appwin.w - 0.5, -0.5, (double) appwin.h - 0.5);
    gr_set_viewport(0, 0, appwin.w, appwin.h);
    redraw = 1;
}

void app_redraw(void)
{
    int i;
    obj_t *obj;

    gr_clear(1.0, 1.0, 1.0, 1.0);
    gr_set_window(-0.5, (double) appwin.w - 0.5, -0.5, (double) appwin.h - 0.5);
    gr_set_viewport(0, 0, appwin.w, appwin.h);
    obj = &displist.objs[0];
    for (i = 0; i < displist.n; i++) {
	obj_draw(obj);
	if (obj->selected) {
	    obj_draw_handles(obj);
	}
	obj++;
    }
    redraw = 0;
}

void app_release1(int x, int y)
{
    int moved;

    y = appwin.h - y - 1;
    moved = (appwin.x1 != x) || (appwin.y1 != y);
    switch (appwin.tool) {
    case TOOL_CURSOR:
	break;
    case TOOL_LINE:
	{
	    obj_t *obj;

	    if (moved && appwin.nclicks1 > 0) {
		obj = &displist.objs[displist.n - 1];
		line_lastpt(obj->data, x, y);
		obj->attr = curattr;
		appwin.nclicks1 = 0;
	    }
	    redraw = 1;
	}
	break;
    default:
	return;
    }
}

void app_selall(void)
{
    int i;

    for (i = 0; i < displist.n; i++) {
	displist.objs[i].selected = 1;
    }
    redraw = 1;
}

void app_tool(int tool)
{
    obj_t *obj;

    appwin.tool = tool;
    if (appwin.nclicks1 > 0) {
	/* the user didn't finish using last tool */
	obj = &displist.objs[displist.n - 1];
	obj_free_data(obj);
	obj->type = OBJ_NONE;
	obj->data = NULL;
	obj->attr = curattr;
	obj->selected = 0;
	displist.n--;

    }
}

void app_usedev(int usedev)
{
    if (usedev) {
	gr_enable_device();
    } else {
	gr_disable_device();
    }
}

void app_back(void)
{
    obj_t tmp;
    int i, j;

    for (i = 1; i < displist.n; i++) {
	if (displist.objs[i].selected) {
	    tmp = displist.objs[i];
	    for (j = i - 1; j >= 0; j--) {
		displist.objs[j + 1] = displist.objs[j];
	    }
	    displist.objs[0] = tmp;
	}
    }
}

void app_lower(void)
{
    obj_t tmp;
    int i;

    for (i = 1; i < displist.n; i++) {
	if (displist.objs[i].selected) {
	    tmp = displist.objs[i];
	    displist.objs[i] = displist.objs[i - 1];
	    displist.objs[i - 1] = tmp;
	}
    }
}

void app_raise(void)
{
    obj_t tmp;
    int i;

    for (i = displist.n - 2; i >= 0; i--) {
	if (displist.objs[i].selected) {
	    tmp = displist.objs[i];
	    displist.objs[i] = displist.objs[i + 1];
	    displist.objs[i + 1] = tmp;
	}
    }
}

void app_front(void)
{
    obj_t tmp;
    int i, j;

    for (i = displist.n - 2; i >= 0; i--) {
	if (displist.objs[i].selected) {
	    tmp = displist.objs[i];
	    for (j = i; j < displist.n - 1; j++) {
		displist.objs[j] = displist.objs[j + 1];
	    }
	    displist.objs[displist.n - 1] = tmp;
	}
    }
}

void dl_move(int src, int dst)
{
    obj_t tmp;

    assert(src >= 0 && src < displist.n);
    assert(dst >= 0 && dst < displist.n);
    tmp = displist.objs[src];
}
