/* $Id: device.c,v 1.4 1997/09/15 07:41:23 wtcorrea Exp $ */

#include <GL/gl.h>
#include <GL/glu.h>
#include <assert.h>
#include "graphics.h"
#include "device.h"

void dev_clear(double r, double g, double b, double a)
{
    glClearColor((float) r, (float) g, (float) b, (float) a);
    glClear(GL_COLOR_BUFFER_BIT);
}

void dev_disable_fill(void)
{
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
}

void dev_draw_line(int x1, int y1, int x2, int y2)
{
    glBegin(GL_LINES);
    glVertex2i(x1, y1);
    glVertex2i(x2, y2);
    glEnd();
}

void dev_draw_polygon(int n, const vertex_t *v)
{
    glBegin(GL_POLYGON);
    while (n--) {
	glVertex2i(v->x, v->y);
	v++;
    }
    glEnd();
}

void dev_draw_polyline(int n, const vertex_t *v)
{
    glBegin(GL_LINE_STRIP);
    while (n--) {
	glVertex2i(v->x, v->y);
	v++;
    }
    glEnd();
}

void dev_draw_triangle(const vertex_t *v)
{
    glBegin(GL_TRIANGLES);
    glVertex2i(v[0].x, v[0].y);
    glVertex2i(v[1].x, v[1].y);
    glVertex2i(v[2].x, v[2].y);
    glEnd();
}

void dev_draw_rect(const vertex_t *v)
{
    glBegin(GL_QUADS);
    glVertex2i(v[0].x, v[0].y);
    glVertex2i(v[1].x, v[0].y);
    glVertex2i(v[1].x, v[1].y);
    glVertex2i(v[0].x, v[1].y);
    glEnd();
}

void dev_enable_fill(void)
{
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

void dev_set_fill_attr(const attr_t *attr)
{
    assert(attr);
    glColor4dv(attr->fcolor);
}

void dev_set_line_attr(const attr_t *attr)
{
    assert(attr);
    glColor4dv(attr->lcolor);
    if (attr->lstippled) {
	glEnable(GL_LINE_STIPPLE);
	glLineStipple(attr->lfactor, attr->lpattern);
    } else {
	glDisable(GL_LINE_STIPPLE);
    }
}

void dev_set_pixel(int x, int y)
{
    glBegin(GL_POINTS);
    glVertex2i(x, y);
    glEnd();
}

void dev_set_viewport(int x, int y, int w, int h)
{
    glViewport(x, y, w, h);
}

void dev_set_window(double l, double r, double b, double t)
{
    glLoadIdentity();
    gluOrtho2D(l, r, b, t);
}
