/* $Id: graphics.h,v 1.8 1997/09/15 14:41:33 wtcorrea Exp $ */

#ifndef GRAPHICS_INCLUDED
#define GRAPHICS_INCLUDED

enum {
    OBJ_NONE,
    OBJ_LINE,
    OBJ_POLYLINE,
    OBJ_POLYGON,
    OBJ_TRIANGLE,
    OBJ_RECT,
    OBJ_CIRCLE,
    OBJ_ELLIPSE,
    OBJ_CURVE,
    OBJ_TEXT
};

typedef struct {
    double lcolor[4];		/* line color */
    int lwidth;			/* line width */
    int lstippled;		/* is line stippled? */
    int lfactor;		/* line stipple factor */
    unsigned short lpattern;	/* line stipple pattern */
    int bounded;		/* is object bounded by a line? */
    int filled;			/* is object filled? */
    double fcolor[4];		/* fill color */
} attr_t;

typedef struct {
    int type;			/* type of object */
    void *data;			/* pointer to object specific data */
    attr_t attr;		/* object attributes */
    int selected;		/* is object selected? */
} obj_t;

typedef struct {
    int x, y;
} vertex_t;

typedef struct {
    vertex_t v[2];
} line_t;

typedef struct {
    vertex_t v[3];
} triangle_t;

typedef struct {
    vertex_t v[2];
} rect_t;

typedef struct {
    vertex_t *v;
    int n, maxn;
} polyline_t;

typedef polyline_t polygon_t;

void obj_free_data(obj_t *obj);
void obj_draw(const obj_t *obj);
void obj_draw_handles(const obj_t *obj);
void obj_move(obj_t *obj, int dx, int dy);
void obj_setbbox(obj_t *obj);
int obj_pick(const obj_t *obj, int x, int y);

line_t *line_new(int x1, int y1, int x2, int y2);
void line_free(line_t *l);
void line_draw(const line_t *l, const attr_t *attr);
void line_draw_handles(const line_t *l);
void line_move(line_t *l, int dx, int dy);
void line_lastpt(line_t *l, int x, int y);
int line_pick(const line_t *l, int x, int y);

polyline_t *polyline_new(int maxn);
polyline_t *polyline_clone(const polyline_t *p);
void polyline_free(polyline_t *p);
void polyline_draw(const polyline_t *p, const attr_t *attr);
void polyline_draw_handles(const polyline_t *p);
void polyline_move(polyline_t *p, int dx, int dy);
void polyline_addpt(polyline_t *p, int x, int y);
void polyline_delpt(polyline_t *p);
void polyline_lastpt(polyline_t *p, int x, int y);
int polyline_npts(const polyline_t *p);
int polyline_pick(const polyline_t *p, int x, int y);

polygon_t *polygon_new(int maxn);
polygon_t *polygon_clone(const polygon_t *p);
void polygon_free(polygon_t *p);
void polygon_draw(const polygon_t *p, const attr_t *attr);
void polygon_draw_handles(const polygon_t *p);
void polygon_move(polygon_t *p, int dx, int dy);
void polygon_addpt(polygon_t *p, int x, int y);
void polygon_delpt(polygon_t *p);
void polygon_lastpt(polygon_t *p, int x, int y);
int polygon_npts(const polygon_t *p);
int polygon_pick(const polygon_t *p, int x, int y);

triangle_t *triangle_new(void);
void triangle_free(triangle_t *t);
void triangle_draw(const triangle_t *t, const attr_t *attr);
void triangle_draw_handles(const triangle_t *t);
void triangle_move(triangle_t *t, int dx, int dy);
void triangle_setv(triangle_t *t, int i, int x, int y);
int triangle_pick(const triangle_t *t, int x, int y);

rect_t *rect_new(int x1, int y1, int x2, int y2);
void rect_free(rect_t * r);
void rect_draw(const rect_t * r, const attr_t *attr);
void rect_draw_handles(const rect_t * r);
void rect_move(rect_t * r, int dx, int dy);
void rect_lastpt(rect_t * r, int x, int y);
int rect_pick(const rect_t * r, int x, int y);

void gr_disable_device(void);
void gr_enable_device(void);
void gr_clear(double r, double g, double b, double a);
void gr_set_viewport(int x, int y, int w, int h);
void gr_set_window(double l, double r, double b, double t);
void gr_draw_handle(int x, int y);
int gr_pt_on_line(int px, int py, int qx, int qy, int tx, int ty);
int gr_pt_in_triang(vertex_t v0, vertex_t v1, vertex_t v2, int x, int y);

#endif
