/* $Id: main.c,v 1.7 1997/09/15 14:52:30 wtcorrea Exp wtcorrea $ */

#include <assert.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#include <tkGLx.h>
#include "app.h"

static int app_init(Tcl_Interp *ti);

static int cb_click1(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_dclick1(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_delete(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_motion(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_new(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_print(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_redraw(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_release1(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_selall(ClientData cd, Tcl_Interp *ti, int argc, char **argv);

static int cb_back(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_lower(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_raise(ClientData cd, Tcl_Interp *ti, int argc, char **argv);
static int cb_front(ClientData cd, Tcl_Interp *ti, int argc, char **argv);

static char *tr_tool(ClientData cd, Tcl_Interp *ti, char *name1, char *name2,
     int flags);
static char *tr_usedev(ClientData cd, Tcl_Interp *ti, char *name1, char *name2,
     int flags);

typedef struct {
    char *name;
    int no;
} tool_t;

static tool_t tools[] = {
    "tool_cursor", TOOL_CURSOR,
    "tool_line", TOOL_LINE,
    "tool_polyline", TOOL_POLYLINE,
    "tool_polygon", TOOL_POLYGON,
    "tool_triangle", TOOL_TRIANGLE,
    "tool_rect", TOOL_RECT,
    "tool_circle", TOOL_CIRCLE,
    "tool_ellipse", TOOL_ELLIPSE,
    "tool_curve", TOOL_CURVE,
    "tool_text", TOOL_TEXT,
    NULL, TOOL_LAST
};

static Tcl_HashTable toolht;

int main(int argc, char *argv[])
{
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;
}

/* Tcl_AppInit() is called from TclMain() after the Tcl interpreter has
 * been created, and before the script file or interactive command loop
 * is entered. */
int Tcl_AppInit(Tcl_Interp *interp)
{
    /* Initialize packages. */

    /* Tcl */
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* Tk */
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* OpenGL */
    if (TkGLx_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* This application */
    if (app_init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* Define a startup filename. This file is read in case the program is
     * run interactively. */
    tcl_RcFileName = ".apprc";

    return TCL_OK;
}

int cb_click1(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_click1(x, y, s);
    return TCL_OK;
}

int cb_dclick1(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_dclick1(x, y, s);
    return TCL_OK;
}

static int cb_delete(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_delete();
    return TCL_OK;
}

static int cb_motion(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y;

    (void) cd;
    if (argc != 3) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    app_motion(x, y);
    return TCL_OK;
}

static int cb_motion1(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y;

    (void) cd;
    if (argc != 3) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    app_motion1(x, y);
    return TCL_OK;
}

static int cb_print(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_print();
    return TCL_OK;
}

static int cb_new(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int w, h;

    (void) cd;
    if (argc != 3) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <w> <h>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &w) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &h) != TCL_OK) {
	return TCL_ERROR;
    }
    app_new(w, h);
    return TCL_OK;
}

static int cb_redraw(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_redraw();
    return TCL_OK;
}

static int cb_release1(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y;

    (void) cd;
    if (argc != 3) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    app_release1(x, y);
    return TCL_OK;
}

static int cb_selall(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_selall();
    return TCL_OK;
}

static int cb_back(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_back();
    return TCL_OK;
}

static int cb_lower(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_lower();
    return TCL_OK;
}

static int cb_raise(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_raise();
    return TCL_OK;
}

static int cb_front(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_front();
    return TCL_OK;
}

static char *tr_tool(ClientData cd, Tcl_Interp *ti, char *name1, char *name2,
     int flags)
{
    int value;
    char *valuestr;

    (void) cd;
    valuestr = Tcl_GetVar2(ti, name1, name2, flags & TCL_GLOBAL_ONLY);
    assert(valuestr);
    value = (int) Tcl_GetHashValue(Tcl_FindHashEntry(&toolht, valuestr));
    assert(value > TOOL_FIRST && value < TOOL_LAST);
    app_tool(value);
    return NULL;
}

static char *tr_usedev(ClientData cd, Tcl_Interp *ti, char *name1, char *name2,
     int flags)
{
    int value;
    char *valuestr;

    (void) cd;
    valuestr = Tcl_GetVar2(ti, name1, name2, flags & TCL_GLOBAL_ONLY);
    assert(valuestr);
    value = atoi(valuestr);
    assert(value == 0 || value == 1);
    app_usedev(value);
    return NULL;
}

int app_init(Tcl_Interp *ti)
{
    tool_t *tp;
    int newentry;
    Tcl_HashEntry *entryp;

    Tcl_CreateCommand(ti, "app_click1", cb_click1, NULL, NULL);
    Tcl_CreateCommand(ti, "app_dclick1", cb_dclick1, NULL, NULL);
    Tcl_CreateCommand(ti, "app_delete", cb_delete, NULL, NULL);
    Tcl_CreateCommand(ti, "app_motion", cb_motion, NULL, NULL);
    Tcl_CreateCommand(ti, "app_motion1", cb_motion1, NULL, NULL);
    Tcl_CreateCommand(ti, "app_new", cb_new, NULL, NULL);
    Tcl_CreateCommand(ti, "app_print", cb_print, NULL, NULL);
    Tcl_CreateCommand(ti, "app_redraw", cb_redraw, NULL, NULL);
    Tcl_CreateCommand(ti, "app_release1", cb_release1, NULL, NULL);
    Tcl_CreateCommand(ti, "app_selall", cb_selall, NULL, NULL);

    Tcl_CreateCommand(ti, "app_back", cb_back, NULL, NULL);
    Tcl_CreateCommand(ti, "app_lower", cb_lower, NULL, NULL);
    Tcl_CreateCommand(ti, "app_raise", cb_raise, NULL, NULL);
    Tcl_CreateCommand(ti, "app_front", cb_front, NULL, NULL);

    Tcl_TraceVar(ti, "tool", TCL_TRACE_WRITES, tr_tool, NULL);
    Tcl_TraceVar(ti, "usedev", TCL_TRACE_WRITES, tr_usedev, NULL);
    Tcl_LinkVar(ti, "redraw", (char *) &redraw, TCL_LINK_INT);

    Tcl_InitHashTable(&toolht, TCL_STRING_KEYS);
    for (tp = tools; tp->name != NULL; tp++) {
	entryp = Tcl_CreateHashEntry(&toolht, tp->name, &newentry);
	Tcl_SetHashValue(entryp, tp->no);
    }
    return TCL_OK;
}
