/* $Id: polygon.c,v 1.3 1997/09/15 14:41:33 wtcorrea Exp $ */

#include <assert.h>
#include "graphics.h"
#include "device.h"
#include "my.h"

extern int usedev;

polygon_t *polygon_new(int maxn)
{
    return polyline_new(maxn);
}

polygon_t *polygon_clone(const polygon_t *p)
{
    return polyline_clone(p);
}

void polygon_free(polygon_t *p)
{
    polyline_free(p);
}

void polygon_draw(const polygon_t *p, const attr_t *attr)
{
    assert(p);
    assert(attr);
    if (usedev) {
	if (attr->filled) {
	    dev_enable_fill();
	    dev_set_fill_attr(attr);
	    dev_draw_polygon(p->n, p->v);
	}
	if (attr->bounded) {
	    dev_disable_fill();
	    dev_set_line_attr(attr);
	    if (p->n > 2) {
		dev_draw_polygon(p->n, p->v);
	    } else {
		dev_draw_polyline(p->n, p->v);
	    }
	}
    } else {
	if (attr->filled) {
	    my_enable_fill();
	    my_set_fill_attr(attr);
	    my_draw_polygon(p->n, p->v);
	}
	if (attr->bounded) {
	    my_disable_fill();
	    my_set_line_attr(attr);
	    if (p->n > 2) {
		my_draw_polygon(p->n, p->v);
	    } else {
		my_draw_polyline(p->n, p->v);
	    }
	}
    }
}

void polygon_draw_handles(const polygon_t *p)
{
    polyline_draw_handles(p);
}

void polygon_move(polygon_t *p, int dx, int dy)
{
    polyline_move(p, dx, dy);
}

void polygon_addpt(polygon_t *p, int x, int y)
{
    polyline_addpt(p, x, y);
}

void polygon_delpt(polygon_t *p)
{
    polyline_delpt(p);
}

void polygon_lastpt(polygon_t *p, int x, int y)
{
    polyline_lastpt(p, x, y);
}

int polygon_npts(const polygon_t *p)
{
    assert(p);
    return p->n;
}

int polygon_pick(const polygon_t *p, int x, int y)
{
    int i;

    for (i = 1; i < p->n - 1; i++) {
	if (gr_pt_in_triang(p->v[0], p->v[i], p->v[i + 1], x, y)) {
	    return 1;
	}
    }
    return 0;
}
