/* $Id: polyline.c,v 1.3 1997/09/15 14:41:33 wtcorrea Exp $ */

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include "graphics.h"
#include "device.h"
#include "my.h"

extern int usedev;

polyline_t *polyline_new(int maxn)
{
    polyline_t *p;

    p = malloc(sizeof(polyline_t));
    assert(p);
    p->v = malloc((size_t) maxn * sizeof(vertex_t));
    assert(p->v);
    p->maxn = maxn;
    p->n = 0;
    return p;
}

polyline_t *polyline_clone(const polyline_t *p)
{
    polyline_t *q;

    assert(p);
    q = polyline_new(p->n);
    q->n = p->n;
    memcpy(q->v, p->v, (size_t) q->n * sizeof(vertex_t));
    return q;
}

void polyline_free(polyline_t *p)
{
    if (p != NULL) {
	free(p->v);
	free(p);
    }
}

void polyline_draw(const polyline_t *p, const attr_t *attr)
{
    assert(p);
    assert(attr);
    if (usedev) {
	dev_set_line_attr(attr);
	dev_draw_polyline(p->n, p->v);
    } else {
	my_set_line_attr(attr);
	my_draw_polyline(p->n, p->v);
    }
}

void polyline_draw_handles(const polyline_t *p)
{
    int i;

    assert(p);
    for (i = 0; i < p->n; i++) {
	gr_draw_handle(p->v[i].x, p->v[i].y);
    }
}

void polyline_move(polyline_t *p, int dx, int dy)
{
    int i;

    assert(p);
    for (i = 0; i < p->n; i++) {
	p->v[i].x += dx, p->v[i].y += dy;
    }
}

void polyline_addpt(polyline_t *p, int x, int y)
{
    assert(p);
    if (p->n < p->maxn) {
	p->v[p->n].x = x, p->v[p->n].y = y;
	p->n++;
    }
}

void polyline_delpt(polyline_t *p)
{
    assert(p);
    assert(p->n > 0);
    p->n--;
}

void polyline_lastpt(polyline_t *p, int x, int y)
{
    assert(p);
    assert(p->n > 0);
    p->v[p->n - 1].x = x, p->v[p->n - 1].y = y;
}

int polyline_npts(const polyline_t *p)
{
    assert(p);
    return p->n;
}

int polyline_pick(const polyline_t *p, int x, int y)
{
    int i;

    for (i = 0; i < p->n - 1; i++) {
	if (gr_pt_on_line(p->v[i].x, p->v[i].y, p->v[i + 1].x, p->v[i + 1].y,
		x, y) == 2) {
	    return 1;
	}
    }
    return 0;
}
