/* $Id: rect.c,v 1.3 1997/09/15 14:41:33 wtcorrea Exp $ */

#include <assert.h>
#include <stdlib.h>
#include "graphics.h"
#include "device.h"
#include "my.h"
#include "macros.h"

extern int usedev;

rect_t *rect_new(int x1, int y1, int x2, int y2)
{
    rect_t *r;

    r = malloc(sizeof(rect_t));
    assert(r);
    r->v[0].x = x1, r->v[0].y = y1, r->v[1].x = x2, r->v[1].y = y2;
    return r;
}

void rect_free(rect_t * r)
{
    if (r != NULL) {
	free(r);
    }
}

void rect_draw(const rect_t * r, const attr_t *attr)
{
    assert(r);
    assert(attr);
    if (usedev) {
	if (attr->filled) {
	    dev_enable_fill();
	    dev_set_fill_attr(attr);
	    dev_draw_rect(r->v);
	}
	if (attr->bounded) {
	    dev_disable_fill();
	    dev_set_line_attr(attr);
	    dev_draw_rect(r->v);
	}
    } else {
	if (attr->filled) {
	    my_enable_fill();
	    my_set_fill_attr(attr);
	    my_draw_rect(r->v);
	}
	if (attr->bounded) {
	    my_disable_fill();
	    my_set_line_attr(attr);
	    my_draw_rect(r->v);
	}
    }
}

void rect_draw_handles(const rect_t * r)
{
    assert(r);
    gr_draw_handle(r->v[0].x, r->v[0].y);
    gr_draw_handle(r->v[1].x, r->v[0].y);
    gr_draw_handle(r->v[1].x, r->v[1].y);
    gr_draw_handle(r->v[0].x, r->v[1].y);
}

void rect_move(rect_t * r, int dx, int dy)
{
    assert(r);
    r->v[0].x += dx, r->v[0].y += dy;
    r->v[1].x += dx, r->v[1].y += dy;
}

void rect_lastpt(rect_t * r, int x, int y)
{
    assert(r);
    r->v[1].x = x, r->v[1].y = y;
}

int rect_pick(const rect_t * r, int x, int y)
{
    assert(r);
    return (x >= min(r->v[0].x, r->v[1].x) - 2) &&
	(x <= max(r->v[0].x, r->v[1].x) + 2) &&
	(y >= min(r->v[0].y, r->v[1].y) - 2) &&
	(y <= max(r->v[0].y, r->v[1].y) + 2);
}
