/* $Id: triangle.c,v 1.3 1997/09/15 14:41:33 wtcorrea Exp $ */

#include <assert.h>
#include <stdlib.h>
#include "graphics.h"
#include "device.h"
#include "my.h"
#include "macros.h"

extern int usedev;

triangle_t *triangle_new(void)
{
    triangle_t *t;

    t = malloc(sizeof(triangle_t));
    assert(t);
    return t;
}

void triangle_free(triangle_t *t)
{
    if (t != NULL) {
	free(t);
    }
}

void triangle_draw(const triangle_t *t, const attr_t *attr)
{
    assert(t);
    assert(attr);
    if (usedev) {
	if (attr->filled) {
	    dev_enable_fill();
	    dev_set_fill_attr(attr);
	    dev_draw_triangle(t->v);
	}
	if (attr->bounded) {
	    dev_disable_fill();
	    dev_set_line_attr(attr);
	    dev_draw_triangle(t->v);
	}
    } else {
	if (attr->filled) {
	    my_enable_fill();
	    my_set_fill_attr(attr);
	    my_draw_triangle(t->v);
	}
	if (attr->bounded) {
	    my_disable_fill();
	    my_set_line_attr(attr);
	    my_draw_triangle(t->v);
	}
    }
}

void triangle_draw_handles(const triangle_t *t)
{
    assert(t);
    gr_draw_handle(t->v[0].x, t->v[0].y);
    gr_draw_handle(t->v[1].x, t->v[1].y);
    gr_draw_handle(t->v[2].x, t->v[2].y);
}

void triangle_move(triangle_t *t, int dx, int dy)
{
    assert(t);
    t->v[0].x += dx, t->v[0].y += dy;
    t->v[1].x += dx, t->v[1].y += dy;
    t->v[2].x += dx, t->v[2].y += dy;
}

void triangle_setv(triangle_t *t, int i, int x, int y)
{
    assert(t);
    assert(i >= 0 && i < 3);
    t->v[i].x = x, t->v[i].y = y;
}

int triangle_pick(const triangle_t *t, int x, int y)
{
    return gr_pt_in_triang(t->v[0], t->v[1], t->v[2], x, y);
}
